# vim: set fileencoding=utf-8 :
#
# (C) 2016,2017 Johannes Hubertz <johannes@hubertz.de>
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, please see
#    <http://www.gnu.org/licenses/>
#
#    Additionally this program is licensed under the terms of the
#    Apache License, Version 2, see the top level directory right here,
#    or you can view there:  <http://www.apache.org/licenses/>
#
from __future__ import absolute_import, division, print_function

__all__ = [
    "__title__", "__author__", "__author_email__",
    "__license__", "__url__", "__version__",
]

__title__ = "iptables-converter"

__author__ = "Johannes Hubertz"
__author_email__ = "johannes@hubertz.de"

__license__ = "GNU General Public License version 3 or later, Apache License Version 2"

__url__ = "https://github.com/sl0/conv.git"

__version__ = "0.9.10"



#!/usr/bin/env python

#encoding:utf8

from iptables_conv.iptables_converter import Chains, Tables, ConverterError
import unittest
import sys

dst = sys.stdout


class Chains_Test(unittest.TestCase):
    '''some tests for class Chain'''

    def test_01_create_a_chain_object(self):
        """
        Chain 01: create a Filter group, f.e. filter
        """
        self.assertIsInstance(Chains("filter",
                                     ["INPUT", "FORWARD", "OUTPUT"]), Chains)
        self.assertEquals({}, Chains("filter", []))
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        self.assertEquals("filter", filter.name)
        self.assertEquals(['INPUT', 'FORWARD', 'OUTPUT'], filter.tables)
        self.assertEquals("-", filter.policy)
        self.assertEquals(0, filter.length)
        self.assertEquals(
            {'FORWARD': 'ACCEPT', 'INPUT': 'ACCEPT', 'OUTPUT': 'ACCEPT'},
            filter.poli)

    def test_02_prove_policies(self):
        """
        Chain 02: check 3 valid policies, 1 exception
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-P INPUT DROP")
        self.assertEquals(
            {'FORWARD': 'ACCEPT', 'INPUT': 'DROP', 'OUTPUT': 'ACCEPT'},
            filter.poli)
        filter.put_into_fgr("-P FORWARD REJECT")
        self.assertEquals(
            {'FORWARD': 'REJECT', 'INPUT': 'DROP', 'OUTPUT': 'ACCEPT'},
            filter.poli)
        filter.put_into_fgr("-P OUTPUT DROP")
        self.assertEquals(
            {'FORWARD': 'REJECT', 'INPUT': 'DROP', 'OUTPUT': 'DROP'},
            filter.poli)
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-P OUTPUT FAIL")

    def test_03_tables_names(self):
        """
        Chain 03: 3 cases OK, 1 Exception
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-t filter -A INPUT -i sl0 -j ACCEPT")
        self.assertEquals(['-A INPUT -i sl0 -j ACCEPT '], filter.data["INPUT"])

        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-t nat -A OUTPUT -j ACCEPT")
        self.assertEquals(['-A OUTPUT -j ACCEPT '], filter.data["OUTPUT"])

        filter.put_into_fgr("-t nat -A FORWARD -j ACCEPT")
        self.assertEquals(['-A FORWARD -j ACCEPT '], filter.data["FORWARD"])

        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-t na -A INPUT")

    def test_04_flush(self):
        """
        Chain 04: flush filter group, 2 rules and an invalid chain
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-t filter -A INPUT -i sl0 -j ACCEPT")
        self.assertEquals(['-A INPUT -i sl0 -j ACCEPT '],
                          filter.data["INPUT"])
        filter.put_into_fgr("-A OUTPUT -o sl1 -j ACCEPT")
        self.assertEquals(['-A OUTPUT -o sl1 -j ACCEPT'],
                          filter.data["OUTPUT"])

        filter.put_into_fgr("-F")
        self.assertEquals([], filter.data["INPUT"])
        self.assertEquals([], filter.data["OUTPUT"])

        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-t inval -F")

    def test_05_new_chain(self):
        """
        Chain 05: create a new chain in filtergroup,
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-N NEWCHAIN")
        self.assertEquals(
            {'FORWARD': [], 'INPUT': [], 'NEWCHAIN': [], 'OUTPUT': []},
            filter.data)

    def test_06_new_existing_chain_fails(self):
        """
        Chain 06: create an exsiting chain should fail
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-N INPUT")

    def test_07_insert_rule_fail(self):
        """
        Chain 07: insert a rule into an empty chain fails
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-I INPUT -j ACCEPT")

    def test_08_insert_rule_fail(self):
        """
        Chain 08: insert a rule into a non_existing chain fails
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-I PUT -j ACCEPT")

    def test_09_insert_rule_works(self):
        """
        Chain 09: insert a rule into a nonempty chain works at start
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-A INPUT -p tcp -j ACCEPT")
        filter.put_into_fgr("-I INPUT -p udp -j ACCEPT")
        filter.put_into_fgr("-I INPUT -p esp -j ACCEPT")
        expect = ['-I INPUT -p esp -j ACCEPT',
                  '-I INPUT -p udp -j ACCEPT',
                  '-A INPUT -p tcp -j ACCEPT']
        self.assertEquals(expect, filter.data["INPUT"])

    def test_10_append_rule(self):
        """
        Chain 10: append a rule to a chain
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-A INPUT -p tcp -j ACCEPT")
        self.assertEquals(['-A INPUT -p tcp -j ACCEPT'], filter.data["INPUT"])
        filter.put_into_fgr("-A INPUT -p udp -j ACCEPT")
        filter.put_into_fgr("-A INPUT -p esp -j ACCEPT")
        expect = ['-A INPUT -p tcp -j ACCEPT',
                  '-A INPUT -p udp -j ACCEPT',
                  '-A INPUT -p esp -j ACCEPT']
        self.assertEquals(expect, filter.data["INPUT"])

    def test_11_remove_predef_chain(self):
        """
        Chain 11: try to remove a prefined chain
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-X INPUT")

    def test_12_remove_chain(self):
        """
        Chain 12: try to remove an existing chain
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        filter.put_into_fgr("-N NEWCHAIN")
        self.assertEquals(
            {'FORWARD': [], 'INPUT': [], 'NEWCHAIN': [], 'OUTPUT': []},
            filter.data)
        filter.put_into_fgr("-X NEWCHAIN")
        self.assertEquals(
            {'FORWARD': [], 'INPUT': [], 'OUTPUT': []},
            filter.data)

    def test_13_illegal_command(self):
        """
        Chain 13: try an ilegal command
        """
        filter = Chains("filter", ["INPUT", "FORWARD", "OUTPUT"])
        with self.assertRaises(ConverterError):
            filter.put_into_fgr("-Y USERCHAIN")


class Tables_Test(unittest.TestCase):
    '''
    Tables: some first tests for the class
    '''

    def test_01_create_a_tables_object(self):
        """
        Tables 01: create a Tables object, check chains
        """
        self.assertIsInstance(Tables(dst, ""), Tables)

        tables = Tables(dst, "")
        expect = {'filter': {'FORWARD': [], 'INPUT': [], 'OUTPUT': []},
                  'raw': {'OUTPUT': [], 'PREROUTING': []},
                  'mangle': {'FORWARD': [], 'INPUT': [],
                             'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []},
                  'nat': {'OUTPUT': [], 'PREROUTING': [], 'POSTROUTING': []}}
        self.assertEquals(expect, tables.data)

    def test_02_nat_prerouting(self):
        """
        Tables 02: nat PREROUTING entry
        """
        tables = Tables(dst, "")
        line = "iptables -t nat -A PREROUTING -s 10.0.0.0/21"
        line = line + " -p tcp --dport   80 -j SNAT --to-source 192.168.1.15"
        tables.put_into_tables(line)
        expect = ['-A PREROUTING -s 10.0.0.0/21 -p tcp --dport 80 -j SNAT --to-source 192.168.1.15 ']
        self.assertEquals(expect, tables.data["nat"]["PREROUTING"])

    def test_03_mangle_table(self):
        """
        Tables 03: mangle INPUT entry
        """
        tables = Tables(dst, "")
        line = "iptables -t mangle -A INPUT"
        line = line + " -p tcp --dport   80 -j ACCEPT"
        tables.put_into_tables(line)
        expect = ['-A INPUT -p tcp --dport 80 -j ACCEPT ']
        self.assertEquals(expect, tables.data["mangle"]["INPUT"])

    def test_04_raw_table(self):
        """
        Tables 04: raw OUTPUT entry
        """
        tables = Tables(dst, "")
        line = "iptables -t raw -A OUTPUT"
        line = line + " -p tcp --dport   80 -j ACCEPT"
        tables.put_into_tables(line)
        expect = ['-A OUTPUT -p tcp --dport 80 -j ACCEPT ']
        self.assertEquals(expect, tables.data["raw"]["OUTPUT"])

    def test_05_not_existing_chain(self):
        """
        Tables 05: INPUT to not existing chain
        """
        line = "iptables -t raw -A NONEXIST -p tcp --dport   80 -j ACCEPT"
        with self.assertRaises(ConverterError):
            Tables(dst, "").put_into_tables(line)

    def test_06_read_not_existing_file(self):
        """
        Tables 06: read non existing file
        """
        with self.assertRaises(ConverterError):
            Tables(dst, "not-exist-is-ok")

    def test_07_read_empty_file(self):
        """
        Tables 07: read empty file (in relation to iptables-commands)
        """
        filename = "MANIFEST.in"
        tables = Tables(dst, filename)
        expect = {'filter': {'FORWARD': [], 'INPUT': [], 'OUTPUT': []},
                  'raw': {'OUTPUT': [], 'PREROUTING': []},
                  'mangle': {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [],
                             'PREROUTING': [], 'OUTPUT': []},
                  'nat': {'OUTPUT': [], 'PREROUTING': [], 'POSTROUTING': []}}
        self.assertEquals(expect, tables.data)

    def test_08_reference_one(self):
        """
        Tables 08: read default file: reference-one, check chains
        """
        tables = Tables(dst)
        expect = {
            'filter': {'FORWARD': [],
                       'INPUT': ['-A INPUT -p tcp --dport 23 -j ACCEPT '],
                       'USER_CHAIN': ['-A USER_CHAIN -p icmp -j DROP '],
                       'OUTPUT': []},
            'raw': {'OUTPUT': [], 'PREROUTING': []},
            'mangle': {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [],
                       'PREROUTING': [], 'OUTPUT': []},
            'nat': {'OUTPUT': [],
                    'POSTROUTING': ['-A POSTROUTING -s 10.0.0.0/21 -p tcp --dport 80 -j SNAT --to-source 192.168.1.15 '],
                    'PREROUTING': ['-A PREROUTING -d 192.0.2.5/32 -p tcp --dport 443 -j DNAT --to-destination 10.0.0.5:1500 ']}}
        self.maxDiff = None
        self.assertEquals(expect, tables.data)

    def test_09_shell_variables(self):
        """
        Tables 09: read buggy file with shell variables
        """
        expect = "Line 8:"
        with self.assertRaisesRegexp(ConverterError, expect):
            Tables(dst, 'tests/data/test-shell-variables')

    def test_10_shell_functions(self):
        """
        Tables 10: read buggy file with shell functions
        """
        expect = "Line 6:"
        with self.assertRaisesRegexp(ConverterError, expect):
            Tables(dst, 'tests/data/test-debian-bug-no-748638')

    def test_11_reference_sloppy_one(self):
        """
        Tables 11: read sloppy input file: reference-sloppy-one, check chains
        """
        tables = Tables(dst, 'reference-sloppy-one', True)
        expect = {
            'filter':
                {'FORWARD': [], 'INPUT': ['-A INPUT -p tcp --dport 23 -j ACCEPT '],
                 'USER_CHAIN': ['-I USER_CHAIN -p icmp --icmp-type echo-request -j ACCEPT ',
                                '-A USER_CHAIN -p icmp --icmp-type echo-reply -j ACCEPT ',
                                '-A USER_CHAIN -p icmp -j DROP '], 'OUTPUT': []},
            'raw': {'OUTPUT': [], 'PREROUTING': []},
            'mangle': {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []},
            'nat': {'OUTPUT': [],
                    'PREROUTING': ['-A PREROUTING -d 192.0.2.5/32 -p tcp --dport 443 -j DNAT --to-destination 10.0.0.5:1500 '],
                    'POSTROUTING': ['-A POSTROUTING -s 10.0.0.0/21 -p tcp --dport 80 -j SNAT --to-source 192.168.1.15 ']}
        }
        self.maxDiff = None
        self.assertEqual(expect, tables.data)

    def test_12_create_a_tables6_object(self):
        """
        Tables 12: create an ipv6 Tables object, check chains
        """
        self.assertIsInstance(Tables(dst, "", ipversion=6), Tables)

        tables = Tables(dst, "", ipversion=6)
        expect = {'filter': {'FORWARD': [], 'INPUT': [], 'OUTPUT': []},
                  'raw': {'OUTPUT': [], 'PREROUTING': []},
                  'mangle': {'FORWARD': [], 'INPUT': [],
                             'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []},
                  'nat': {'OUTPUT': [], 'PREROUTING': [], 'POSTROUTING': []}}
        self.assertEquals(expect, tables.data)

    def test_13_re6ference_one(self):
        """
        Tables 13: read default file: re6ference-one, check chains
        """
        tables = Tables(dst, "re6ference-one", ipversion=6)
        expect = {
            'filter': {'FORWARD': [],
                       'INPUT': ['-A INPUT -p tcp --dport 23 -j ACCEPT '],
                       'USER_CHAIN': ['-A USER_CHAIN -p icmp -j DROP '], 'OUTPUT': []},
            'raw': {'OUTPUT': [], 'PREROUTING': []},
            'mangle': {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []},
            'nat': {'OUTPUT': [], 'PREROUTING': ['-A PREROUTING -d 2001:db8:feed::1/128 -p tcp --dport 443 -j DNAT --to-destination 2001:db8:feed::1:1500 '], 'POSTROUTING': ['-A POSTROUTING -s 2001:db8:dead::/64 -p tcp --dport 80 -j SNAT --to-source 2001:db8:feed::1 ']}
        }
        self.maxDiff = None
        self.assertEquals(expect, tables.data)

    def test_14_re6ference_sloppy_one(self):
        """
        Tables 14: read sloppy input file: re6ference-sloppy-one, check chains
        """
        tables = Tables(dst, 're6ference-sloppy-one', sloppy=True, ipversion=6)
        expect = {
            'filter':
                {'FORWARD': [], 'INPUT': ['-A INPUT -p tcp --dport 23 -j ACCEPT '],
                 'USER_CHAIN': ['-A USER_CHAIN -p icmp -j DROP '], 'OUTPUT': []},
            'raw': {'OUTPUT': [], 'PREROUTING': []},
            'mangle': {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []},
            'nat': {'OUTPUT': [],
                    'PREROUTING':
                    ['-A PREROUTING -d 2001:db8:feed::1/128 -p tcp --dport 443 -j DNAT --to-destination 2001:db8:feed::1:1500 '],
                    'POSTROUTING':
                    ['-A POSTROUTING -s 2001:db8:dead::/64 -p tcp --dport 80 -j SNAT --to-source 2001:db8:feed::1 ']}
        }
        self.maxDiff = None
        self.assertEqual(expect, tables.data)


def test_15_tables_printout(capsys):
    """
    Tables 15: check table_printout as well
    """
    tables = Tables(sys.stdout, 'reference-one')
    tables.table_printout()
    out, err = capsys.readouterr()
    assert len(err) == 0
    words = ['*raw', '*nat', '*mangle', '*filter', 'COMMIT', 'from:',
             'INPUT', 'FORWARD', 'USER_CHAIN', '192.0.2.5', ]
    absents = ['iptables', '-t raw', '-t mangle', 'udp', ]
    for word in words:
        assert word in out
    for absent in absents:
        assert absent not in out


if __name__ == "__main__":
        unittest.main()



import pytest
import sys
from iptables_conv.iptables_converter import my_options, main


def test_01_iptables_converter_option_h(capsys):
    """ check ip6tablesmy_options returns help text
    """
    sys.argv = ['ip6tables-converter', '-h', ]
    with pytest.raises(SystemExit) as execinfo:
        options = my_options()
        print(':' + str(options) + ':')

    out, err = capsys.readouterr()

    assert ': SystemExit' in str(execinfo)

    items = ['Options', 'SOURCEFILE', 'DESTFILE', 'version',
             '--help', '--sloppy', 'Have Fun!', ]

    for item in items:
        assert item in out

    assert len(err) == 0


def test_02_iptables_converter_option_s():
    """ check my_options sloppy and sourcefile name returning
    """
    sys.argv = ['iptab', '-s', 'reference-one', '--sloppy', ]
    options = my_options()

    assert options == {'destfile': None,
                       'sourcefile': 'reference-one',
                       'sloppy': True}


def test_03_iptables_converter_option_d():
    """ check my_options, destfile name returning
    """
    sys.argv = ['ip6tab', '-d', 'one', ]
    options = my_options()

    assert options == {'destfile': 'one',
                       'sourcefile': None,
                       'sloppy': False}


def test_04_iptables_converter_option_sd():
    """ check my_options returns source- and destfile
    """
    sys.argv = ['bla', '-s', 'one', '-d', 'two']
    options = my_options()

    assert options == {'destfile': 'two',
                       'sourcefile': 'one',
                       'sloppy': False}


def test_05_iptables_converter_main_dlft(capsys):
    """ check main behavior on default file not found
    """
    sys.argv = ['ip6tab', ]
    result = main()
    out, err = capsys.readouterr()

    assert 'Errno 2' in err
    assert 'No such file or directory:' in err
    assert 'rules' in err
    assert result == 1


def test_06_iptables_converter_main_ok(capsys):
    """ check main reads a named file
    """
    sys.argv = ['ip6tab', '-s', 'reference-one', ]
    result = main()
    out, err = capsys.readouterr()

    # print(':' + out + ':')
    assert '# generated from: reference-one' in out
    assert result == 0


def test_07_iptables_converter_main_write(capsys):
    """ check main reads and writes w/o error
    """
    sys.argv = ['ip6tab', '-s', 'reference-one', '-d', '/dev/null', ]
    result = main()
    out, err = capsys.readouterr()

    assert len(out) == 0
    assert len(err) == 0
    assert result == 0



#
# -*- coding: utf-8 -*-
#
"""
iptables_converter.py:
    convert iptables commands within a script
    into a correspondig iptables-save script

    default filename to read is rules, to read some other
        file, append: -s filename

    default output is written to stdout, for writing
        to some file, append: -d filename

Author:     Johannes Hubertz <johannes@hubertz.de>
Date:       2017-11-16
Version:    see __init__.version
License:    GNU General Public License version 3 or later
            Apache License Version 2.0

Have Fun!
"""

from __future__ import print_function

try:
    from collections import UserDict
except ImportError:
    from UserDict import UserDict
from optparse import OptionParser
import re
import os
import sys
from .__init__ import __version__


class ConverterError(Exception):
    pass


class Chains(UserDict):
    """ chains are grouped in iptables

    :param str name: chain group name, 'filter', 'nat', ...
    :param list tables: list of chains
    :param bool sloppy: needs '-N'(default) or not

    :return: object representing chain group
    :rtype: Chains
    :raises ConverterError: on some illegal conditions
    """

    def __init__(self, name, tables, sloppy=False):
        """ init Chains object
        """
        UserDict.__init__(self)
        self.name = name
        self.tables = tables
        self.predef = tables
        self.sloppy = sloppy
        self.reset()  # name, tables)

    def put_into_fgr(self, content):
        """ fill this line into this tabular

        :param str content: one line of inputfile
        :return: None
        :raises ConverterError: on some illegal conditions
        """
        self.length += 1
        if len(content) == 0:
            return
        elements = content.split()
        action = elements[0]
        if "-t" in action:
            elements.pop(0)  # remove 1st: -t
            fname = elements.pop(0)
            legals = ["filter", "nat", "raw", "mangle"]
            if fname not in legals:
                msg = "Valid is one of %s, got: %s" % (legals, fname)
                raise ConverterError(msg)
            action = elements[0]
            content = ""                       # rebuild content from here
            for elem in elements:
                content = content + elem + " "
            if len(elements) > 1:
                chain_name = elements[1]
        if "-F" in action:
            self.reset()
            return
        if "-P" in action:
            elements.pop(0)
            cha = elements.pop(0)
            new = elements.pop(0)
            if new not in ["ACCEPT", "DROP", "REJECT"]:
                msg = "Illegal policy: % s" % (new)
                raise ConverterError(msg)
            self.poli[cha] = new
            return
        if "-X" in action:
            predef = ['INPUT', 'FORWARD', 'OUTPUT',
                      'PREROUTING', 'POSTROUTING']
            rem_chain_name = elements.pop(1)
            if rem_chain_name in predef:
                msg = "Cannot remove predefined chain"
                raise ConverterError(msg)
            if rem_chain_name in self.data:
                self.data[rem_chain_name] = []        # empty list
                self.poli[rem_chain_name] = "-"       # empty policy, no need
                self.data.pop(rem_chain_name)
            return
        if "-N" in action:
            new_chain_name = elements.pop(1)
            existing = self.data.keys()
            if new_chain_name in existing:
                msg = "Chain %s already exists" % (new_chain_name)
                raise ConverterError(msg)
            self.data[new_chain_name] = []        # empty list
            self.poli[new_chain_name] = "-"       # empty policy, no need
            return
        if "-I" in action:
            chain_name = elements[1]
            existing = self.data.keys()
            if chain_name not in existing:
                msg = "invalid chain name: %s" % (chain_name)
                if not self.sloppy:
                    raise ConverterError(msg)
                else:
                    new_chain_name = elements[1]
                    self.data[new_chain_name] = []
                    self.poli[new_chain_name] = '-'
            stuff = self.data[chain_name]
            if len(stuff) > 0:
                stuff.insert(0, content)
            else:
                msg = "Empty chain %s allows append only!" % (chain_name)
                raise ConverterError(msg)
            self.data[chain_name] = stuff
            return
        if "-A" in action:
            chain_name = elements[1]
            existing = self.data.keys()
            if chain_name not in existing:
                msg = "invalid chain name: %s" % (chain_name)
                if not self.sloppy:
                    raise ConverterError(msg)
                else:
                    new_chain_name = elements[1]
                    self.data[new_chain_name] = []
                    self.poli[new_chain_name] = '-'
            stuff = self.data[chain_name]
            stuff.append(content)
            self.data[chain_name] = stuff
            return
        msg = "Unknown filter command in input:" + content
        raise ConverterError(msg)

    def reset(self):
        """ action method for iptables -F
        """
        self.poli = {}               # empty dict
        self.length = 0
        self.policy = "-"
        for tabular in self.tables:
            self.data[tabular] = []
            self.poli[tabular] = "ACCEPT"


class Tables(UserDict):
    """ some chaingroups in tables are predef: filter, nat, mangle, raw

    :param str destfile: which file or pathname is to be written
    :param str sourcefile: which file or pathname is to be read
    :param bool sloppy: '-N' is needed(default) or not
    :param int ipversion: 4(default) or 6

    :return: Tables after read sourcefile written into destfile
    :rtype: Tables

    :raises ConverterError: on some illegal conditions
    """

    def __init__(self,
                 destfile,
                 sourcefile="reference-one",
                 sloppy=False,
                 ipversion=4
                 ):
        """init Tables Object is easy going"""
        UserDict.__init__(self)
        self.destfile = destfile
        self.sourcefile = sourcefile
        self.sloppy = sloppy
        self.linecounter = 0
        self.tblctr = 0
        self.patterns = ""
        self.reset(sourcefile, ipversion)

    def reset(self, sourcefile, ipversion):
        """all predefined Chains aka lists are setup as new here

        :param str sourcefile: file to be read
        :param int ipversion: 4 or 6

        :return: None
        """
        self.patterns = ['^iptables', '^/sbin/iptables', ]
        if ipversion == 6:
            self.patterns = ['^ip6tables', '^/sbin/ip6tables', ]

        filt = ["INPUT", "FORWARD", "OUTPUT"]
        filters = Chains("filter", filt, self.sloppy)

        mang = ["PREROUTING", "INPUT", "FORWARD", "OUTPUT", "POSTROUTING", ]
        mangle = Chains("mangle", mang, self.sloppy)

        # kernel 2.6.32 has no INPUT in NAT!
        nats = ["PREROUTING", "OUTPUT", "POSTROUTING"]
        nat = Chains("nat", nats, self.sloppy)

        raws = ["PREROUTING", "OUTPUT", ]
        raw = Chains("raw", raws, self.sloppy)

        self.data["filter"] = filters
        self.data["mangle"] = mangle
        self.data["nat"] = nat
        self.data["raw"] = raw
        if len(sourcefile) > 0:
            self.linecounter = self.read_file(sourcefile)

    def table_printout(self):
        """printout nonempty tabulars in fixed sequence"""
        self.destfile.write("# generated from: %s\n" % (self.sourcefile))
        for key in ["raw", "nat", "mangle", "filter"]:
            count = self.data[key].length
            if count > -1:
                self.destfile.write("*%s\n" % (self.data[key].name))
                for chain in self.data[key].keys():
                    poli = self.data[key].poli[chain]
                    self.destfile.write(":%s %s [0:0]\n" % (chain, poli))
                for chain in self.data[key].values():
                    for elem in chain:
                        self.destfile.write(elem)
                        self.destfile.write('\n')
                self.destfile.write("COMMIT\n")

    def put_into_tables(self, line):
        """put line into matching Chains-object

        :param line: one line of inputfile
        :return: None
        """
        elements = line.split()
        elements.pop(0)                     # we always know, it's iptables
        rest = ""
        for elem in elements:               # remove redirects and the like
            if ">" not in elem:
                rest = rest + elem + " "    # string again with single blanks
        fam = "filter"
        if "-t nat" in line:                # nat filter group
            fam = "nat"
        elif "-t mangle" in line:           # mangle filter group
            fam = "mangle"
        elif "-t raw" in line:              # raw filter group
            fam = "raw"
        fam_dict = self.data[fam]           # select the group dictionary
        fam_dict.put_into_fgr(rest)         # do action thers

    def read_file(self, sourcefile):
        """ open file or error

        :param sourcefile: file or pathname of file to be read

        :return: file_descriptor
        :raises ConverterError: on IOError
        """
        try:
            with open(sourcefile, 'r') as file_descriptor:
                return self.read(file_descriptor)
        except IOError as err:
            raise ConverterError(str(err))

    def read(self, file_descriptor):
        """read data from file like object into Tables-object

        :param file_descriptor: filedescriptor of file to be read

        :return: None
        :raises ConverterError: on some illegal conditions
        """
        self.linecounter = 0
        self.tblctr = 0
        try:
            for this_line in file_descriptor:
                line = str(this_line.strip())
                self.linecounter += 1
                if line.startswith('#'):
                    continue
                for element in ['\$', '\(', '\)', ]:
                    if re.search(element, line):
                        mstart = "Line %d:\n%s\nplain files only, " % \
                            (self.linecounter, line)
                        if element in ['\(', '\)', ]:
                            merr = "unable to convert shell functions, abort"
                        else:
                            merr = "unable to resolve shell variables, abort"
                        msg = mstart + merr
                        raise ConverterError(msg)
                for pattern in self.patterns:
                    if re.search(pattern, line):
                        self.tblctr += 1
                        self.put_into_tables(line)
        except ValueError as err:
            raise ConverterError(str(err))


def my_options():
    """ commandline options assembly

    :returns: :class: OptionParser
    """
    version = "version %s" % __version__
    usage = "usage:  %prog --help | -h \n\n\t%prog: " + version
    usage = usage + "\tHave Fun!"
    parser = OptionParser(usage)
    parser.disable_interspersed_args()
    parser.add_option("-d", "", dest="destfile",
                      type="string",
                      help="output filename, default: stdout\n")
    parser.add_option("-s", "", dest="sourcefile",
                      type="string",
                      help="file with iptables commands, default: rules\n")
    parser.add_option("--sloppy", "", dest="sloppy",
                      action="store_true", default=False,
                      help="-N name-of-userchain is inserted automatically,\n"
                           "by default -N is neccessary in input\n")
    (options, _) = parser.parse_args()
    return options


def main():
    """ main parses options, filnames and the like
    option -s needs input-filename to be read,
    if it is not given, it defaults to: rules.
    option -d needs output-filename to be written,
    if it is not given, it defaults to: sys.stdout
    """

    options = my_options()

    if options.sourcefile is None:
        options.sourcefile = "rules"
    sourcefile = options.sourcefile

    if options.destfile is not None:
        destfile = open(options.destfile, 'w')
    else:
        destfile = sys.stdout

    ipversion = 4
    if os.path.basename(sys.argv[0]).startswith("ip6"):
        ipversion = 6

    try:
        chains = Tables(destfile,
                        sourcefile,
                        options.sloppy,
                        ipversion=ipversion
                        )
        chains.table_printout()
    except ConverterError as err:
        print(str(err), file=sys.stderr)
        return 1
    finally:
        if destfile != sys.stdout:
            destfile.close()
    return 0



""" setup.py for iptables-converter
"""

import os
import sys
import iptables_conv

from setuptools import setup, find_packages


if sys.argv[-1] == 'test':
    dev_requirements = [
        'pytest',
        'flake8',
        'coverage'
    ]
    try:
        _ = map(__import__, dev_requirements)
    except ImportError as err:
        err_msg = err.message.replace("No module named ", "")
        msg = "%s is not installed. Install your test requirments." % err_msg
        raise ImportError(msg)
    os.system('flake8 iptables_conv tests')
    os.system('py.test')
    sys.exit()


setup(name='iptables-converter',
      description='convert set of iptables-commands to iptables-save format',
      long_description=open('README.rst').read(),
      version=iptables_conv.__version__,
      license=iptables_conv.__license__,
      author=iptables_conv.__author__,
      author_email=iptables_conv.__author_email__,
      url=iptables_conv.__url__,
      platforms=['Linux', ],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: '
          'GNU General Public License v3 or later (GPLv3+)',
          'License :: OSI Approved :: Apache Software License',
          'Intended Audience :: System Administrators',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Topic :: Security',
          'Topic :: System :: Networking :: Firewalls',
          'Topic :: Utilities',
      ],
      packages=find_packages(exclude=['tests', ]),
      entry_points={
          'console_scripts': [
              'iptables-converter=iptables_conv.iptables_converter:main',
              'ip6tables-converter=iptables_conv.iptables_converter:main',
          ],
      },
     )
